#
# ImageCrop V1.0a
# Copyright 2002 Satoshi Imai
#
package require Img
package require BLT
set cur [file dirname [info script]]
set auto_path [linsert $auto_path 0 [file join $cur img_rotate1.1]]
package require img_rotate

set title ImageCrop
set scale_height 160
set scale_width 120
set sample_width 640
set sample_height 480
set jpeg_quality 100
set scale_x 100
set scale_y 100
set opendir {}
set savedir {}

wm resizable . no no
wm title . ImageCrop

pack [frame .bar] -fill x
button .bar.open -text "J..." -command OpenFile
button .bar.save -text "ۑ..." -command SaveFile
button .bar.up -text "gg" -command ScaleUp
button .bar.down -text "gk" -command ScaleDown
button .bar.rotater -text "E]" -command Rotate_R
button .bar.rotatel -text "]" -command Rotate_L
label .bar.label1 -text "TCY c: "
entry .bar.height -width 4 -textvariable scale_height
label .bar.label2 -text ": "
entry .bar.width -width 4 -textvariable scale_width
label .bar.label3 -text "JPEGi: "
entry .bar.quality -width 4 -textvariable jpeg_quality
button .bar.close -text  -command exit
pack .bar.open .bar.save .bar.up .bar.down .bar.rotater .bar.rotatel \
	.bar.label1 .bar.height .bar.label2 .bar.width .bar.label3 .bar.quality -side left
pack .bar.close -side right
pack [canvas .c -height $sample_height -width $sample_width -bg black]
.c create rectangle $scale_x $scale_y [expr $scale_x+$scale_width] [expr $scale_y+$scale_height] \
	-outline yellow -width 2 -tag scale

bind . <Up> ScaleUp
bind . <Down> ScaleDown
bind .c <ButtonPress-1> {CanvasMark %x %y %W}
bind .c <ButtonRelease-1> {CanvasClear %W}
bind .c <Motion> {catch {CanvasDrag %x %y %W}}
bind . <Control-o> OpenFile
bind . <Control-s> SaveFile
bind .bar.height <Return> ChangeScale
bind .bar.width <Return> ChangeScale
bind .bar.quality <Return> ChangeQuality

proc ImageView {} {
	global sample_width sample_height scale_x scale_y scale_width scale_height scale
	if {[image height main] < [image width main]} {
		set iw $sample_width
		if {[image width main] < $iw} {
			set iw [image width main]
		}
		set scale [expr $iw.0 / [image width main].0]
		set ih [expr int([image height main] * $scale)]
	} else {
		set ih $sample_height
		if {[image height main] < $ih} {
			set ih [image height main]
		}
		set scale [expr $ih.0 / [image height main].0]
		set iw [expr int([image width main] * $scale)]
	}
	set scale [expr [image height main].0/$ih.0]
	image create photo sample -width $iw -height $ih
	::blt::winop resample main sample sinc
	.c delete all
	.c configure -height $ih -width $iw
	.c create image 0 0 -image sample -anchor nw
	set scale_x 100
	set scale_y 100
	.c create rectangle $scale_x $scale_y [expr $scale_x+$scale_width] [expr $scale_y+$scale_height] \
	-outline yellow -width 2 -tag scale
}

proc Rotate_R {} {
	if {[catch {image type main}]} return
	set cursor [. cget -cursor]
	. configure -cursor watch
	update
	::img_rotate::img_rotate main R
	ImageView
	. configure -cursor $cursor
}

proc Rotate_L {} {
	if {[catch {image type main}]} return
	set cursor [. cget -cursor]
	. configure -cursor watch
	update
	::img_rotate::img_rotate main L
	ImageView
	. configure -cursor $cursor
}

proc ChangeScale {} {
	global scale_x scale_y scale_width scale_height
	if {![string is digit $scale_width] || $scale_width == ""} {
		set scale_width 120
	}
	if {![string is digit $scale_height] || $scale_height == ""} {
		set scale_height 160
	}
	.c delete scale
	.c create rectangle $scale_x $scale_y [expr $scale_x+$scale_width] [expr $scale_y+$scale_height] \
	-outline yellow -width 2 -tag scale
}

proc ChangeQuality {} {
	global jpeg_quality
	if {![string is digit $jpeg_quality] || $jpeg_quality == ""} {
		set jpeg_quality 100
	}
}

proc OpenFile {} {
	global sample_width sample_height scale_x scale_y scale_width scale_height scale title opendir
	set types {
		{ "Jpeg File" 	{.jpg .jpeg .JPG .JPEG} }
		{ "Bmp File" 	{.bmp .BMP} }
		{ "Gif File" 	{.gif .GIF} }
		{ "All File" 	{*} }
    }
	set file [tk_getOpenFile -filetypes $types -title J -initialdir $opendir]
	if { $file == "" } return
	set opendir [file dirname $file]
	image create photo main -file $file
	ImageView
	wm title . "$title $file"
}

proc SaveFile {} {
	global scale_height scale_width jpeg_quality savedir
	set types {
		{ "Jpeg File" 	{.jpg} }
		{ "Bmp File" 	{.bmp} }
	}
	set file [tk_getSaveFile -filetypes $types -title ۑ -initialdir $savedir -defaultextension ".jpg .bmp"]
	if {$file == ""} {
	    return
	}
	set savedir [file dirname $file]
	image create photo temp1
	image create photo temp2 -height $scale_height -width $scale_width
	if {[catch "temp1 copy main -from [coordinate_int .c] -to 0 0"]} {
		tk_messageBox -title "G[" -icon info \
			-message "ۑ͈̔͂C[W͈̔͂𒴂Ă܂B\n͈͐ݒ蒼ĂB"
		image delete temp1 temp2
		return
	}
	::blt::winop resample temp1 temp2 sinc
	set ext [file extension $file]
	if {[string equal -nocase $ext ".jpg"] || [string equal -nocase $ext ".jpeg"]} {
		temp2 write $file -format "jpeg -quality $jpeg_quality" ;#  -optimize -smooth 30
	} else {
		temp2 write $file -format "bmp"
	}
	image delete temp1 temp2
}

proc ScaleUp {} {
	global scale_x scale_y
	.c scale scale $scale_x $scale_y 1.1 1.1
}

proc ScaleDown {} {
	global scale_x scale_y
	.c scale scale $scale_x $scale_y 0.9 0.9
}

proc coordinate_int {w} {
	global scale
	set xy [$w coords scale]
	set x1 [expr int([lindex $xy 0]*$scale)]
	set y1 [expr int([lindex $xy 1]*$scale)]
	set x2 [expr int([lindex $xy 2]*$scale)]
	set y2 [expr int([lindex $xy 3]*$scale)]
	return "$x1 $y1 $x2 $y2"
}

proc coordinate_fmt {w} {
	set xy [$w coords scale]
	set x1 [expr int([lindex $xy 0])]
	set y1 [expr int([lindex $xy 1])]
	set x2 [expr int([lindex $xy 2])]
	set y2 [expr int([lindex $xy 3])]
	return [format "(%s, %s) - (%s, %s)" $x1 $y1 $x2 $y2]
}

proc CanvasMark {x y w} {
	global canvas
	set canvas(obj) 1
	set canvas(x) $x
	set canvas(y) $y
	set canvas(txt) [.c create text [expr $x+20] $y -text [coordinate_fmt $w] -fill yellow -anchor w -tag text]
}

proc CanvasClear {w} {
	global canvas
	set canvas(obj) 0
	$w delete text
}

proc CanvasDrag {x y w} {
	global canvas scale_x scale_y
	if {$canvas(obj) == 0} return
	set dx [expr $x - $canvas(x)]
	set dy [expr $y - $canvas(y)]
	set canvas(x) $x
	set canvas(y) $y
	$w move scale $dx $dy
	set scale_x [lindex [$w coords scale] 0]
	set scale_y [lindex [$w coords scale] 1]
	$w itemconfigure text -text [coordinate_fmt $w]
	$w move text $dx $dy
}
